@@ -11,7 +11,7 @@ from pychess.Utils.book import getOpenings
 from pychess.Utils.const import NORMALCHESS, FEN_START, BLACK, FISCHERRANDOMCHESS, \
     CRAZYHOUSECHESS, WILDCASTLESHUFFLECHESS, LOSERSCHESS, SUICIDECHESS, ATOMICCHESS, \
     THREECHECKCHESS, KINGOFTHEHILLCHESS, ASEANCHESS, MAKRUKCHESS, CAMBODIANCHESS, \
-    SITTUYINCHESS, GIVEAWAYCHESS, HORDECHESS, WHITE
+    SITTUYINCHESS, GIVEAWAYCHESS, HORDECHESS, RACINGKINGSCHESS, PLACEMENTCHESS, WHITE
 from pychess.Utils.lutils.Benchmark import benchmark
 from pychess.Utils.lutils.perft import perft
 from pychess.Utils.lutils.LBoard import LBoard
@@ -22,6 +22,7 @@ from pychess.Utils.lutils.lmovegen import genAllMoves, genCaptures, genCheckEvas
 from pychess.Utils.lutils.validator import validateMove
 from pychess.System.Log import log
 from pychess.Variants.horde import HORDESTART
+from pychess.Variants.placement import PLACEMENTSTART
 from pychess.Variants.asean import ASEANSTART, MAKRUKSTART, KAMBODIANSTART, SITTUYINSTART
 
 if sys.platform != "win32":
@@ -45,8 +46,6 @@ class PyChessCECP(PyChess):
         self.analyzing = False
         self.thread = None
 
-        self.basetime = 0
-
         self.features = {
             "ping": 1,
             "setboard": 1,
@@ -61,8 +60,8 @@ class PyChessCECP(PyChess):
             "analyze": 1,
             "myname": "PyChess %s" % pychess.VERSION,
             "variants": "normal,wildcastle,nocastle,fischerandom,crazyhouse," +
-                        "losers,suicide,giveaway,horde,atomic," +
-                        "kingofthehill,3check,asean,cambodian,makruk,sittuyin",
+                        "losers,suicide,giveaway,horde,atomic,racingkings," +
+                        "kingofthehill,3check,placement,asean,cambodian,makruk,sittuyin",
             "colors": 0,
             "ics": 0,
             "name": 0,
@@ -151,6 +150,8 @@ class PyChessCECP(PyChess):
                             self.board.iniAtomic()
                         elif lines[1] == "3check":
                             self.board.variant = THREECHECKCHESS
+                        elif lines[1] == "racingkings":
+                            self.board.variant = RACINGKINGSCHESS
                         elif lines[1] == "kingofthehill":
                             self.board.variant = KINGOFTHEHILLCHESS
                             self.print("setup (PNBRQKpnbrqk) 8x8+0_fairy %s" %
@@ -158,6 +159,9 @@ class PyChessCECP(PyChess):
                         elif lines[1] == "horde":
                             self.board = LBoard(HORDECHESS)
                             self.board.applyFen(HORDESTART)
+                        elif lines[1] == "placement":
+                            self.board = LBoard(PLACEMENTCHESS)
+                            self.board.applyFen(PLACEMENTSTART)
                         elif lines[1] == "asean":
                             self.board = LBoard(ASEANCHESS)
                             self.board.applyFen(ASEANSTART)
@@ -229,7 +233,8 @@ class PyChessCECP(PyChess):
                     if len(minutes) > 1 and minutes[1]:
                         self.basetime += int(minutes[1])
                     self.clock[:] = self.basetime, self.basetime
-                    self.increment = inc, inc
+                    self.increment = inc
+                    self.searchtime = 0
 
                 elif lines[0] == "st":
                     self.searchtime = float(lines[1])
@@ -350,7 +355,10 @@ class PyChessCECP(PyChess):
 
                 elif lines[0] == "egtpath":
                     if len(lines) >= 3 and lines[1] == "gaviota":
-                        conf.set("egtb_path", conf.get("egtb_path", lines[2]))
+                        if lines[2]:
+                            conf.set("egtb_path", lines[2])
+                        else:
+                            conf.set("egtb_path", conf.get("egtb_path"))
                         from pychess.Utils.lutils.lsearch import enableEGTB
                         enableEGTB()
 
