@@ -1,40 +1,56 @@
 # -*- coding: UTF-8 -*-
-
+import re
 
 from sqlalchemy import select, func, and_, or_
 
 from pychess.Utils.const import FEN_START, WHITE, BLACK, reprResult
 from pychess.Database import model as dbmodel
-from pychess.Database.model import game, event, site, player, pl1, pl2, annotator, source
+from pychess.Database.model import game, event, site, player, pl1, pl2, annotator, source, tag_game
 
 
 count_games = select([func.count()]).select_from(game)
 
 
+def parseDateTag(tag):
+    elements = re.match("^([0-9\?]{4})(\.([0-9\?]{2})(\.([0-9\?]{2}))?)?$", tag)
+    if elements is None:
+        y, m, d = None, None, None
+    else:
+        elements = elements.groups()
+        try:
+            y = int(elements[0])
+        except Exception:
+            y = None
+        try:
+            m = int(elements[2])
+        except Exception:
+            m = None
+        try:
+            d = int(elements[4])
+        except Exception:
+            d = None
+    return y, m, d
+
+
 def save(path, model, offset):
     game_event = model.tags["Event"]
     game_site = model.tags["Site"]
 
-    year, month, day = int(model.tags["Year"]), int(model.tags["Month"]), int(model.tags["Day"])
-    game_round = model.tags["Round"] if "Round" in model.tags else ""
+    date = model.tags["Date"]
+
+    game_round = model.tags["Round"]
 
     white = repr(model.players[WHITE])
     black = repr(model.players[BLACK])
 
     result = model.status
-    eco = model.tags["ECO"] if "ECO" in model.tags else ""
+    eco = model.tags["ECO"]
 
-    time_control = model.tags["TimeControl"] if "TimeControl" in model.tags else ""
+    time_control = model.tags["TimeControl"]
     board = int(model.tags["Board"]) if "Board" in model.tags else 0
 
-    if "WhiteElo" in model.tags:
-        white_elo = int(model.tags["WhiteElo"]) if model.tags["WhiteElo"] else 0
-    else:
-        white_elo = 0
-    if "BlackElo" in model.tags:
-        black_elo = int(model.tags["BlackElo"]) if model.tags["BlackElo"] else 0
-    else:
-        black_elo = 0
+    white_elo = model.tags["WhiteElo"]
+    black_elo = model.tags["BlackElo"]
 
     variant = model.variant.variant
 
@@ -72,9 +88,7 @@ def save(path, model, offset):
             'offset8': (offset >> 3) << 3,
             'event_id': event_id,
             'site_id': site_id,
-            'date_year': year,
-            'date_month': month,
-            'date_day': day,
+            'date': date,
             'round': game_round,
             'white_id': white_id,
             'black_id': black_id,
@@ -113,9 +127,7 @@ col2label = {game.c.id: "Id",
              event.c.name: "Event",
              site.c.name: "Site",
              game.c.round: "Round",
-             game.c.date_year: "Year",
-             game.c.date_month: "Month",
-             game.c.date_day: "Day",
+             game.c.date: "Date",
              game.c.white_elo: "WhiteElo",
              game.c.black_elo: "BlackElo",
              game.c.ply_count: "PlyCount",
@@ -170,23 +182,23 @@ class TagDatabase:
             tags = []
             if "white" in tag_query:
                 if "ignore_tag_colors" in tag_query:
-                    tags.append(or_(pl1.c.name.startswith(tag_query["white"]),
-                                    pl2.c.name.startswith(tag_query["white"])))
+                    tags.append(or_(pl1.c.name.like("%%%s%%" % tag_query["white"]),
+                                    pl2.c.name.like("%%%s%%" % tag_query["white"])))
                 else:
-                    tags.append(pl1.c.name.startswith(tag_query["white"]))
+                    tags.append(pl1.c.name.like("%%%s%%" % tag_query["white"]))
 
             if "black" in tag_query:
                 if "ignore_tag_colors" in tag_query:
-                    tags.append(or_(pl1.c.name.startswith(tag_query["black"]),
-                                    pl2.c.name.startswith(tag_query["black"])))
+                    tags.append(or_(pl1.c.name.like("%%%s%%" % tag_query["black"]),
+                                    pl2.c.name.like("%%%s%%" % tag_query["black"])))
                 else:
-                    tags.append(pl2.c.name.startswith(tag_query["black"]))
+                    tags.append(pl2.c.name.like("%%%s%%" % tag_query["black"]))
 
             if "event" in tag_query:
-                tags.append(event.c.name.startswith(tag_query["event"])),
+                tags.append(event.c.name.like("%%%s%%" % tag_query["event"])),
 
             if "site" in tag_query:
-                tags.append(site.c.name.startswith(tag_query["site"])),
+                tags.append(site.c.name.like("%%%s%%" % tag_query["site"])),
 
             if "eco_from" in tag_query:
                 tags.append(game.c.eco >= tag_query["eco_from"])
@@ -195,21 +207,40 @@ class TagDatabase:
                 tags.append(game.c.eco <= tag_query["eco_to"])
 
             if "annotator" in tag_query:
-                tags.append(annotator.c.name.startswith(tag_query["annotator"])),
+                tags.append(annotator.c.name.like("%%%s%%" % tag_query["annotator"])),
+
+            if "variant" in tag_query:
+                tags.append(game.c.variant == int(tag_query["variant"])),
 
             if "result" in tag_query:
                 tags.append(game.c.result == reprResult.index(tag_query["result"])),
 
-            if "year_from" in tag_query:
-                tags.append(game.c.date_year >= tag_query["year_from"])
+            if "date_from" in tag_query:
+                tags.append(game.c.date >= tag_query["date_from"])
 
-            if "year_to" in tag_query:
-                tags.append(game.c.date_year <= tag_query["year_to"])
+            if "date_to" in tag_query:
+                # When date_to is not given as full date we have to prepare
+                # date_to filled with some "?" to get correct query results
+                # because for example "2018.??.??" is greater than "2018"
+                date_to = tag_query["date_to"]
+                y, m, d = parseDateTag(date_to)
+                y = "%04d" % y if y is not None else "????"
+                m = "%02d" % m if m is not None else "??"
+                d = "%02d" % d if d is not None else "??"
+                date_to = "%s.%s.%s" % (y, m, d)
+
+                tags.append(game.c.date <= date_to)
 
             if "elo_from" in tag_query:
                 tags.append(game.c.white_elo >= tag_query["elo_from"])
                 tags.append(game.c.black_elo >= tag_query["elo_from"])
 
+                # white_elo and black_elo are String(4) in game table
+                # we need to prevent selecting games where a player elo is 999 or 400
+                # when tag_query["elo_from"] is for example was set to 2200
+                tags.append(game.c.white_elo < "4")
+                tags.append(game.c.black_elo < "4")
+
             if "elo_to" in tag_query:
                 tags.append(game.c.white_elo <= tag_query["elo_to"])
                 tags.append(game.c.black_elo <= tag_query["elo_to"])
@@ -278,3 +309,6 @@ class TagDatabase:
             return None
         else:
             return result[0]
+
+    def get_exta_tags(self, rec):
+        return self.engine.execute(select([tag_game]).where(tag_game.c.game_id == rec["Id"]))
