@@ -10,7 +10,7 @@ from .Cord import Cord
 from .Move import Move
 from .const import LOSERSCHESS, WHITE, WHITEWON, BLACKWON, WON_NOMATERIAL, KING, HORDECHESS, \
     SUICIDECHESS, GIVEAWAYCHESS, ATOMICCHESS, WON_KINGEXPLODE, KINGOFTHEHILLCHESS, BLACK, DRAW, \
-    WON_KINGINCENTER, THREECHECKCHESS, WON_THREECHECK, WON_MATE, DRAW_STALEMATE, \
+    CRAZYHOUSECHESS, WON_KINGINCENTER, THREECHECKCHESS, WON_THREECHECK, WON_MATE, DRAW_STALEMATE, \
     DRAW_INSUFFICIENT, DRAW_EQUALMATERIAL, WON_LESSMATERIAL, WON_WIPEOUT, DRAW_REPITITION, \
     WON_KINGINEIGHTROW, RACINGKINGSCHESS, DRAW_50MOVES, RUNNING, ENPASSANT, UNKNOWN_REASON
 
@@ -43,6 +43,8 @@ def isClaimableDraw(board):
 
 
 def playerHasMatingMaterial(board, playercolor):
+    if board.variant == CRAZYHOUSECHESS:
+        return True
     lboard = board.board
     return ldraw.testPlayerMatingMaterial(lboard, playercolor)
 
