@@ -1,6 +1,7 @@
 import asyncio
 from io import StringIO
 
+from pychess.compat import create_task
 from pychess.System.Log import log
 from pychess.Utils.GameModel import GameModel
 from pychess.Utils.Offer import Offer
@@ -289,13 +290,14 @@ class ICGameModel(GameModel):
     ############################################################################
 
     def onKibitzMessage(self, cm, name, gameno, text):
+        @asyncio.coroutine
         def coro():
             if not self.gmwidg_ready.is_set():
                 yield from self.gmwidg_ready.wait()
             if gameno != self.ficsgame.gameno:
                 return
             self.emit("message_received", name, text)
-        self.kibitz_task = asyncio.async(coro())
+        self.kibitz_task = create_task(coro())
 
     def onWhisperMessage(self, cm, name, gameno, text):
         if gameno != self.ficsgame.gameno:
