@@ -48,8 +48,8 @@ class Perspective(object):
 
             menu_item = Gtk.CheckMenuItem(label=panel.__title__)
             menu_item.name = panel.__name__
-            if menu_item.name != "LecturesPanel":
-                menu_item.set_active(True)
+            # if menu_item.name != "LecturesPanel":
+            #    menu_item.set_active(True)
             menu_item.connect("toggled", self.on_toggled, panel)
             self.menuitems.append(menu_item)
             panel.menu_item = menu_item
@@ -80,6 +80,7 @@ class Perspective(object):
 
         if active:
             self.notebooks[name].show()
+            leaf.setCurrentPanel(name)
             if shown == 0 and hasattr(leaf, "position"):
                 # If this is the first one, adjust Gtk.Paned divider handle
                 if leaf.position != 0:
@@ -97,13 +98,23 @@ class Perspective(object):
                 else:
                     parent.set_position(parent.props.max_position)
 
+    def activate_panel(self, name):
+        for panel in self.sidePanels:
+            if panel.__name__.startswith(name):
+                if panel.menu_item.get_active():
+                    # if menu item is already active set_active() doesn't triggers on_toggled()
+                    self.on_toggled(panel.menu_item, panel)
+                else:
+                    panel.menu_item.set_active(True)
+                break
+
     def load_from_xml(self):
         if os.path.isfile(self.dockLocation):
             try:
                 self.dock.loadFromXML(self.dockLocation, self.docks)
             except Exception as e:
                 # We don't send error message when error caused by no more existing SwitcherPanel
-                if e.args[0] != "SwitcherPanel":
+                if e.args[0] != "SwitcherPanel" and "unittest" not in sys.modules.keys():
                     stringio = StringIO()
                     traceback.print_exc(file=stringio)
                     error = stringio.getvalue()
@@ -200,6 +211,8 @@ class PerspectiveManager(object):
             self.activate_perspective("database")
         elif self.get_perspective("games").sensitive:
             self.activate_perspective("games")
+        elif self.get_perspective("learn").sensitive:
+            self.activate_perspective("learn")
         else:
             self.activate_perspective("welcome")
 
