@@ -1,10 +1,16 @@
+import asyncio
 import os
 import threading
+from struct import pack
 
 from gi.repository import Gtk, GObject, GLib
 
-from pychess.Utils.const import FIRST_PAGE, NEXT_PAGE
+from pychess.Utils.const import FIRST_PAGE, NEXT_PAGE, FEN_START, DRAW, WHITEWON, BLACKWON  # , reprCord
 from pychess.Utils.IconLoader import load_icon
+from pychess.Utils.lutils.LBoard import LBoard
+from pychess.Utils.lutils.lmove import toPolyglot  # , FCORD, TCORD
+from pychess.Utils.GameModel import GameModel
+from pychess.Variants import name2variant, NormalBoard
 from pychess.perspectives import Perspective, perspective_manager
 from pychess.perspectives.database.gamelist import GameList
 from pychess.perspectives.database.OpeningTreePanel import OpeningTreePanel
@@ -18,11 +24,12 @@ from pychess.widgets import gamewidget
 from pychess.Database.model import create_indexes, drop_indexes
 from pychess.Database.PgnImport import PgnImport, download_file
 from pychess.Database.JvR import JvR
-from pychess.Savers import fen, epd
+from pychess.Savers import fen, epd, olv
 from pychess.Savers.pgn import PGNFile
 from pychess.System.protoopen import protoopen
 
 pgn_icon = load_icon(24, "application-x-chess-pgn", "pychess")
+BOOK_DEPTH = 8
 
 
 class Database(GObject.GObject, Perspective):
@@ -31,6 +38,7 @@ class Database(GObject.GObject, Perspective):
         'chessfile_opened': (GObject.SignalFlags.RUN_FIRST, None, (object, )),
         'chessfile_closed': (GObject.SignalFlags.RUN_FIRST, None, ()),
         'chessfile_imported': (GObject.SignalFlags.RUN_FIRST, None, (object, )),
+        'bookfile_created': (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
     def __init__(self):
@@ -99,14 +107,14 @@ class Database(GObject.GObject, Perspective):
         self.spinner = Gtk.Spinner()
         self.spinner.set_size_request(50, 50)
         self.progressbar0 = Gtk.ProgressBar(show_text=True)
-        self.progressbar1 = Gtk.ProgressBar(show_text=True)
+        self.progressbar = Gtk.ProgressBar(show_text=True)
 
         self.progress_dialog = Gtk.Dialog("", mainwindow(), 0, (
             Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL))
         self.progress_dialog.set_deletable(False)
         self.progress_dialog.get_content_area().pack_start(self.spinner, True, True, 0)
         self.progress_dialog.get_content_area().pack_start(self.progressbar0, True, True, 0)
-        self.progress_dialog.get_content_area().pack_start(self.progressbar1, True, True, 0)
+        self.progress_dialog.get_content_area().pack_start(self.progressbar, True, True, 0)
         self.progress_dialog.get_content_area().show_all()
 
         # Initing headbook
@@ -173,9 +181,19 @@ class Database(GObject.GObject, Perspective):
         self.import_button.set_sensitive(on)
         self.widgets["import_chessfile"].set_sensitive(on)
         self.widgets["database_save_as"].set_sensitive(on)
+        self.widgets["create_book"].set_sensitive(on)
         self.widgets["import_endgame_nl"].set_sensitive(on)
         self.widgets["import_twic"].set_sensitive(on)
 
+        if on:
+            gamewidget.getWidgets()["copy_pgn"].set_property('sensitive', on)
+            gamewidget.getWidgets()["copy_fen"].set_property('sensitive', on)
+        else:
+            persp = perspective_manager.get_perspective("games")
+            if persp.cur_gmwidg() is None:
+                gamewidget.getWidgets()["copy_pgn"].set_property('sensitive', on)
+                gamewidget.getWidgets()["copy_fen"].set_property('sensitive', on)
+
     def open_chessfile(self, filename):
         if self.first_run:
             self.init_layout()
@@ -184,18 +202,16 @@ class Database(GObject.GObject, Perspective):
         perspective_manager.activate_perspective("database")
 
         self.progress_dialog.set_title(_("Open"))
-        self.progressbar0.hide()
         self.spinner.show()
         self.spinner.start()
 
         def opening():
             if filename.endswith(".pgn"):
-                GLib.idle_add(self.progressbar1.show)
-                GLib.idle_add(self.progressbar1.set_text, _("Opening chessfile..."))
-                chessfile = PGNFile(protoopen(filename), self.progressbar1)
-                self.importer = PgnImport(chessfile)
-                chessfile.init_tag_database(self.importer)
-                if self.importer.cancel:
+                GLib.idle_add(self.progressbar.show)
+                GLib.idle_add(self.progressbar.set_text, _("Opening chessfile..."))
+                chessfile = PGNFile(protoopen(filename), self.progressbar)
+                self.importer = chessfile.init_tag_database()
+                if self.importer is not None and self.importer.cancel:
                     chessfile.tag_database.close()
                     if os.path.isfile(chessfile.sqlite_path):
                         os.remove(chessfile.sqlite_path)
@@ -206,6 +222,9 @@ class Database(GObject.GObject, Perspective):
             elif filename.endswith(".epd"):
                 self.importer = None
                 chessfile = epd.load(protoopen(filename))
+            elif filename.endswith(".olv"):
+                self.importer = None
+                chessfile = olv.load(protoopen(filename, encoding="utf-8"))
             elif filename.endswith(".fen"):
                 self.importer = None
                 chessfile = fen.load(protoopen(filename))
@@ -214,6 +233,7 @@ class Database(GObject.GObject, Perspective):
                 chessfile = None
 
             GLib.idle_add(self.spinner.stop)
+            GLib.idle_add(self.spinner.hide)
             GLib.idle_add(self.progress_dialog.hide)
 
             if chessfile is not None:
@@ -282,6 +302,7 @@ class Database(GObject.GObject, Perspective):
                 break
 
         if len(self.chessfiles) == 0:
+            self.chessfile = None
             self.set_sensitives(False)
             perspective_manager.disable_perspective("database")
 
@@ -339,16 +360,25 @@ class Database(GObject.GObject, Perspective):
 
         dialog.destroy()
 
-        if filename is not None:
+        if filename is None:
+            return
+
+        self.progress_dialog.set_title(_("Save as"))
+
+        def save_as(cancel_event):
             with open(filename, "w") as to_file:
-                records, plys = self.chessfile.get_records(FIRST_PAGE)
-                self.save_records(records, to_file)
-                while True:
-                    records, plys = self.chessfile.get_records(NEXT_PAGE)
-                    if records:
-                        self.save_records(records, to_file)
-                    else:
-                        break
+                self.process_records(self.save_records, cancel_event, to_file)
+
+            GLib.idle_add(self.progress_dialog.hide)
+
+        cancel_event = threading.Event()
+        loop = asyncio.get_event_loop()
+        loop.run_in_executor(None, save_as, cancel_event)
+
+        response = self.progress_dialog.run()
+        if response == Gtk.ResponseType.CANCEL:
+            cancel_event.set()
+        self.progress_dialog.hide()
 
     def save_records(self, records, to_file):
         f = self.chessfile.handle
@@ -409,17 +439,17 @@ class Database(GObject.GObject, Perspective):
         self.importer = PgnImport(self.chessfile, append_pgn=True)
         self.importer.initialize()
         for i, filename in enumerate(filenames):
-            GLib.idle_add(self.progressbar0.set_fraction, i / float(len(filenames)))
-            # GLib.idle_add(self.progressbar0.set_text, filename)
+            if len(filenames) > 1:
+                GLib.idle_add(self.progressbar0.set_fraction, i / float(len(filenames)))
             if self.importer.cancel:
                 break
             if isinstance(filename, tuple):
                 info_link, pgn_link = filename
-                self.importer.do_import(pgn_link, info=info_link, progressbar=self.progressbar1)
+                self.importer.do_import(pgn_link, info=info_link, progressbar=self.progressbar)
             else:
-                self.importer.do_import(filename, progressbar=self.progressbar1)
+                self.importer.do_import(filename, progressbar=self.progressbar)
 
-        GLib.idle_add(self.progressbar1.set_text, _("Recreating indexes..."))
+        GLib.idle_add(self.progressbar.set_text, _("Recreating indexes..."))
 
         # .sqlite
         create_indexes(self.chessfile.engine)
@@ -435,22 +465,135 @@ class Database(GObject.GObject, Perspective):
         self.chessfile.set_scout_filter(None)
         GLib.idle_add(self.gamelist.load_games)
         GLib.idle_add(self.emit, "chessfile_imported", self.chessfile)
+        GLib.idle_add(self.progressbar0.hide)
         GLib.idle_add(self.progress_dialog.hide)
 
     def do_import(self, filenames):
         self.progress_dialog.set_title(_("Import"))
-        self.spinner.hide()
-        if len(filenames) == 1:
-            self.progressbar0.hide()
-        else:
+        if len(filenames) > 1:
             self.progressbar0.show()
-        self.progressbar1.show()
-        self.progressbar1.set_text(_("Preparing to start import..."))
+        self.progressbar.show()
+        self.progressbar.set_text(_("Preparing to start import..."))
 
         thread = threading.Thread(target=self.importing, args=(filenames, ))
         thread.daemon = True
         thread.start()
 
+    def process_records(self, callback, cancel_event, *args):
+        counter = 0
+
+        records, plys = self.chessfile.get_records(FIRST_PAGE)
+        callback(records, *args)
+        GLib.idle_add(self.progressbar.set_text, _("%s games processed") % counter)
+
+        while not cancel_event.is_set():
+            records, plys = self.chessfile.get_records(NEXT_PAGE)
+            if records:
+                callback(records, *args)
+                counter += len(records)
+                GLib.idle_add(self.progressbar.set_text, _("%s games processed") % counter)
+            else:
+                break
+
+    def create_book(self, new_bin=None):
+        if new_bin is None:
+            dialog = Gtk.FileChooserDialog(
+                _("Create New Polyglot Opening Book"), mainwindow(), Gtk.FileChooserAction.SAVE,
+                (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_NEW, Gtk.ResponseType.ACCEPT))
+
+            dialog.set_current_folder(os.path.expanduser("~"))
+            dialog.set_current_name("new_book.bin")
+
+            response = dialog.run()
+            if response == Gtk.ResponseType.ACCEPT:
+                new_bin = dialog.get_filename()
+                if not new_bin.endswith(".bin"):
+                    new_bin = "%s.bin" % new_bin
+            dialog.destroy()
+
+        if new_bin is None:
+            return
+
+        self.progress_dialog.set_title(_("Create Polyglot Book"))
+
+        def creating_book(cancel_event):
+            positions = {}
+            self.process_records(self.feed_book, cancel_event, positions)
+
+            if cancel_event.is_set():
+                return
+
+            with open(new_bin, "wb") as to_file:
+                GLib.idle_add(self.progressbar.set_text, _("Save"))
+                for key, moves in sorted(positions.items(), key=lambda item: item[0]):
+                    # print(key, moves)
+                    for move in moves:
+                        to_file.write(pack(">QHHI", key, move, moves[move], 0))
+            GLib.idle_add(self.emit, "bookfile_created")
+            GLib.idle_add(self.progress_dialog.hide)
+
+        cancel_event = threading.Event()
+        loop = asyncio.get_event_loop()
+        loop.run_in_executor(None, creating_book, cancel_event)
+
+        response = self.progress_dialog.run()
+        if response == Gtk.ResponseType.CANCEL:
+            cancel_event.set()
+        self.progress_dialog.hide()
+
+    def feed_book(self, records, positions):
+        for rec in records:
+            model = GameModel()
+
+            if rec["Result"] == DRAW:
+                score = (1, 1)
+            elif rec["Result"] == WHITEWON:
+                score = (2, 0)
+            elif rec["Result"] == BLACKWON:
+                score = (0, 2)
+            else:
+                score = (0, 0)
+
+            fenstr = rec["FEN"]
+            variant = self.chessfile.get_variant(rec)
+
+            if variant:
+                model.variant = name2variant[variant]
+                board = LBoard(model.variant.variant)
+            else:
+                model.variant = NormalBoard
+                board = LBoard()
+
+            if fenstr:
+                try:
+                    board.applyFen(fenstr)
+                except SyntaxError as err:
+                    continue
+            else:
+                board.applyFen(FEN_START)
+
+            boards = [board]
+
+            movetext = self.chessfile.get_movetext(rec)
+            boards = self.chessfile.parse_movetext(movetext, boards[0], -1)
+
+            for board in boards:
+                if board.plyCount > BOOK_DEPTH:
+                    break
+                move = board.lastMove
+                if move is not None:
+                    poly_move = toPolyglot(board.prev, move)
+                    # move_str = "%s%s" % (reprCord[FCORD(move)], reprCord[TCORD(move)])
+                    # print("%0.16x" % board.prev.hash, poly_move, board.prev.asFen(), move_str)
+                    if board.prev.hash in positions:
+                        if poly_move in positions[board.prev.hash]:
+                            positions[board.prev.hash][poly_move] += score[board.prev.color]
+                        else:
+                            positions[board.prev.hash][poly_move] = score[board.prev.color]
+                    else:
+                        # board.prev.asFen(), move_str,
+                        positions[board.prev.hash] = {poly_move: score[board.prev.color]}
+
     def create_database(self):
         dialog = Gtk.FileChooserDialog(
             _("Create New Pgn Database"), mainwindow(), Gtk.FileChooserAction.SAVE,
@@ -475,8 +618,7 @@ class Database(GObject.GObject, Perspective):
                                       buttons=Gtk.ButtonsType.OK)
                 d.set_markup(_("<big><b>File '%s' already exists.</b></big>") % new_pgn)
                 d.run()
-                d.hide()
-                print("%s allready exist." % new_pgn)
+                d.destroy()
 
         dialog.destroy()
 
@@ -497,7 +639,7 @@ class Database(GObject.GObject, Perspective):
         name, ext = os.path.splitext(chessfile.path)
         basename = os.path.basename(name)
         info = "%s.%s" % (basename, ext[1:])
-        tooltip = "%s\ncontaining %s games" % (chessfile.path, chessfile.count)
+        tooltip = _("%(path)s\ncontaining %(count)s games") % ({"path": chessfile.path, "count": chessfile.count})
         tabcontent.set_tooltip_text(tooltip)
 
         label = Gtk.Label(info)
