@@ -6,6 +6,7 @@ from io import StringIO
 
 from gi.repository import GLib, Gtk, GObject
 
+from pychess.compat import create_task
 from pychess.ic import IC_POS_EXAMINATING, IC_POS_OBSERVING_EXAMINATION, \
     get_infobarmessage_content, get_infobarmessage_content2, TITLES
 from pychess.ic.ICGameModel import ICGameModel
@@ -328,9 +329,9 @@ class FICS(GObject.GObject, Perspective):
 
         perspective = perspective_manager.get_perspective("games")
         if not ficsgame.board.fen:
-            asyncio.async(perspective.generalStart(gamemodel, player0tup, player1tup))
+            create_task(perspective.generalStart(gamemodel, player0tup, player1tup))
         else:
-            asyncio.async(perspective.generalStart(gamemodel, player0tup, player1tup, (
+            create_task(perspective.generalStart(gamemodel, player0tup, player1tup, (
                 StringIO(ficsgame.board.fen), fen, 0, -1)))
 
     def onGameModelStarted(self, gamemodel, ficsgame):
@@ -358,7 +359,7 @@ class FICS(GObject.GObject, Perspective):
             bplayer.long_name())
 
         perspective = perspective_manager.get_perspective("games")
-        asyncio.async(perspective.generalStart(gamemodel, player0tup, player1tup, (
+        create_task(perspective.generalStart(gamemodel, player0tup, player1tup, (
             StringIO(ficsgame.board.pgn), pgn, 0, -1)))
 
         if ficsgame.relation == IC_POS_OBSERVING_EXAMINATION:
