@@ -12,12 +12,13 @@ from gi.repository import Gtk
 from gi.repository import GObject
 
 from pychess.Savers.ChessFile import LoadingError
-from pychess.Savers import chessalpha2, epd, fen, pgn, png, database
+from pychess.Savers import epd, fen, pgn, olv, png, database
 from pychess.System import conf
 from pychess.System.Log import log
 from pychess.System.protoopen import isWriteable
 from pychess.System.uistuff import GladeWidgets
 from pychess.System.prefix import addUserConfigPrefix
+from pychess.Savers.pgn import parseDateTag
 from pychess.Utils.const import UNFINISHED_STATES, ABORTED, ABORTED_AGREEMENT, LOCAL, ARTIFICIAL, MENU_ITEMS
 from pychess.Utils.Offer import Offer
 from pychess.widgets import gamewidget, mainwindow, new_notebook
@@ -29,7 +30,7 @@ from pychess.widgets.pydock.__init__ import CENTER, EAST, SOUTH
 from pychess.ic.ICGameModel import ICGameModel
 
 enddir = {}
-savers = (pgn, epd, fen, png, chessalpha2)
+savers = (pgn, epd, fen, olv, png)  # chessalpha2 is broken
 
 saveformats = Gtk.ListStore(str, str, GObject.TYPE_PYOBJECT)
 exportformats = Gtk.ListStore(str, str, GObject.TYPE_PYOBJECT)
@@ -101,7 +102,7 @@ class Games(GObject.GObject, Perspective):
 
         # Initing players
 
-        def set_name(none, player, key, alt):
+        def xxxset_name(none, player, key, alt):
             player.setName(conf.get(key, alt))
 
         players = []
@@ -122,10 +123,10 @@ class Games(GObject.GObject, Perspective):
                 # issue tracker, we need to give human players special treatment
                 player = func(gmwidg, *args)
                 players.append(player)
-
+        assert len(players) == 2
         if player0tup[0] == ARTIFICIAL and player1tup[0] == ARTIFICIAL:
 
-            def emit_action(board, action, param, gmwidg):
+            def emit_action(board, action, player, param, gmwidg):
                 if gmwidg.isInFront():
                     gamemodel.curplayer.emit("offer", Offer(action, param=param))
 
@@ -137,12 +138,19 @@ class Games(GObject.GObject, Perspective):
         log.debug("Games.generalStart: <- gamemodel.setPlayers(): %s" %
                   (gamemodel))
 
+        # Forward information from the engines
+        for playertup, tagname in ((player0tup, "WhiteElo"), (player1tup, "BlackElo")):
+            if playertup[0] == ARTIFICIAL:
+                elo = playertup[2][0].get("elo")
+                if elo:
+                    gamemodel.tags[tagname] = elo
+
         # Starting
         if loaddata:
             try:
                 uri, loader, gameno, position = loaddata
                 gamemodel.loadAndStart(uri, loader, gameno, position)
-                if position != gamemodel.ply:
+                if position != gamemodel.ply and position != -1:
                     gmwidg.board.view.shown = position
             except LoadingError as e:
                 d = Gtk.MessageDialog(mainwindow(), type=Gtk.MessageType.WARNING,
@@ -150,7 +158,7 @@ class Games(GObject.GObject, Perspective):
                 d.set_markup(_("<big><b>Error loading game</big></b>"))
                 d.format_secondary_text(", ".join(str(a) for a in e.args))
                 d.show()
-                d.hide()
+                d.destroy()
 
         else:
             if gamemodel.variant.need_initial_board:
@@ -186,16 +194,20 @@ class Games(GObject.GObject, Perspective):
         game.save(uri, saver, append=False, position=position)
 
     def saveGamePGN(self, game):
-        if conf.get("saveOwnGames", False) and not game.hasLocalPlayer():
+        if conf.get("saveOwnGames") and not game.hasLocalPlayer():
             return True
-        filename = conf.get("autoSaveFormat", "pychess")
+
+        filename = conf.get("autoSaveFormat")
         filename = filename.replace("#n1", game.tags["White"])
         filename = filename.replace("#n2", game.tags["Black"])
-        filename = filename.replace("#y", "%s" % game.tags["Year"])
-        filename = filename.replace("#m", "%s" % game.tags["Month"])
-        filename = filename.replace("#d", "%s" % game.tags["Day"])
-        pgn_path = conf.get("autoSavePath", os.path.expanduser("~")) + \
-            "/" + filename + ".pgn"
+        year, month, day = parseDateTag(game.tags["Date"])
+        year = '' if year is None else str(year)
+        month = '' if month is None else str(month)
+        day = '' if day is None else str(day)
+        filename = filename.replace("#y", "%s" % year)
+        filename = filename.replace("#m", "%s" % month)
+        filename = filename.replace("#d", "%s" % day)
+        pgn_path = conf.get("autoSavePath") + "/" + filename + ".pgn"
         append = True
         try:
             if not os.path.isfile(pgn_path):
@@ -213,7 +225,7 @@ class Games(GObject.GObject, Perspective):
             if scoutfish_path is not None:
                 pgn_text = pgn.save(StringIO(), game)
 
-                tmp = tempfile.NamedTemporaryFile(mode="w", delete=False)
+                tmp = tempfile.NamedTemporaryFile(mode="w", encoding="utf-8", delete=False)
                 pgnfile = tmp.name
                 with tmp.file as f:
                     f.write(pgn_text)
@@ -272,7 +284,7 @@ class Games(GObject.GObject, Perspective):
                         "Was unable to save '%(uri)s' as PyChess doesn't know the format '%(ending)s'.") %
                         {'uri': uri, 'ending': ending})
                     d.run()
-                    d.hide()
+                    d.destroy()
                     continue
                 else:
                     saver = enddir[ending]
@@ -290,7 +302,7 @@ class Games(GObject.GObject, Perspective):
                     "You don't have the necessary rights to save the file.\n\
     Please ensure that you have given the right path and try again."))
                 d.run()
-                d.hide()
+                d.destroy()
                 continue
 
             if os.path.isfile(uri):
@@ -306,7 +318,7 @@ class Games(GObject.GObject, Perspective):
                 d.format_secondary_text(_(
                     "The file already exists in '%s'. If you replace it, its content will be overwritten.") % folder)
                 replaceRes = d.run()
-                d.hide()
+                d.destroy()
 
                 if replaceRes == 1:
                     append = True
@@ -326,7 +338,7 @@ class Games(GObject.GObject, Perspective):
                 d.format_secondary_text(_("The error was: %s") % ", ".join(
                     str(a) for a in e.args))
                 d.run()
-                d.hide()
+                d.destroy()
                 continue
 
             break
@@ -357,7 +369,7 @@ class Games(GObject.GObject, Perspective):
                 if not gmwidg.gamemodel.isChanged():
                     response = Gtk.ResponseType.OK
                 else:
-                    if conf.get("autoSave", False):
+                    if conf.get("autoSave"):
                         x = self.saveGamePGN(game)
                         if x:
                             response = Gtk.ResponseType.OK
@@ -440,7 +452,7 @@ class Games(GObject.GObject, Perspective):
             response = Gtk.ResponseType.OK
         else:
             markup = "<b><big>" + _("Save the current game before you close it?") + "</big></b>"
-            if conf.get("autoSave", False):
+            if conf.get("autoSave"):
                 x = self.saveGamePGN(gmwidg.gamemodel)
                 if x:
                     response = Gtk.ResponseType.OK
@@ -491,8 +503,11 @@ class Games(GObject.GObject, Perspective):
             self.gamewidgets.remove(gmwidg)
             gmwidg.gamemodel.terminate()
 
+            db_persp = perspective_manager.get_perspective("database")
             if len(self.gamewidgets) == 0:
                 for widget in MENU_ITEMS:
+                    if widget in ("copy_pgn", "copy_fen") and db_persp.preview_panel is not None:
+                        continue
                     gamewidget.getWidgets()[widget].set_property('sensitive', False)
 
         return response
@@ -525,7 +540,7 @@ class Games(GObject.GObject, Perspective):
         for notebook in self.notebooks.values():
             notebook.remove_page(pageNum)
 
-        if headbook.get_n_pages() == 1 and conf.get("hideTabs", False):
+        if headbook.get_n_pages() == 1 and conf.get("hideTabs"):
             self.show_tabs(False)
 
         if headbook.get_n_pages() == 0:
@@ -557,7 +572,7 @@ class Games(GObject.GObject, Perspective):
         headbook.set_scrollable(True)
         align.add(headbook)
         perspective_widget.pack_start(align, False, True, 0)
-        self.show_tabs(not conf.get("hideTabs", False))
+        self.show_tabs(not conf.get("hideTabs"))
 
         # Initing center
 
@@ -713,7 +728,7 @@ class Games(GObject.GObject, Perspective):
         if not head:
             return
         if head.get_n_pages() == 1:
-            self.show_tabs(not conf.get("hideTabs", False))
+            self.show_tabs(not conf.get("hideTabs"))
 
     def attachGameWidget(self, gmwidg):
         log.debug("attachGameWidget: %s" % gmwidg)
@@ -757,7 +772,7 @@ class Games(GObject.GObject, Perspective):
         gmwidg.stat_hbox.show_all()
 
         if headbook.get_n_pages() == 1:
-            self.show_tabs(not conf.get("hideTabs", False))
+            self.show_tabs(not conf.get("hideTabs"))
         else:
             # We should always show tabs if more than one exists
             self.show_tabs(True)
