@@ -3,13 +3,16 @@
 
 from math import floor, ceil, pi
 from time import time
+from io import StringIO
 
 import cairo
 from gi.repository import GLib, Gtk, Gdk, GObject, Pango, PangoCairo
 
+from pychess.Savers import pgn
 from pychess.System.prefix import addDataPrefix
 from pychess.System import conf
 from pychess.gfx import Pieces
+from pychess.Savers.pgn import comment_arrows_re, comment_circles_re
 from pychess.Utils.Cord import Cord
 from pychess.Utils.GameModel import GameModel
 from pychess.Utils.const import ASEAN_VARIANTS, DROP_VARIANTS, WAITING_TO_START, REMOTE, \
@@ -18,7 +21,6 @@ from pychess.Utils.const import ASEAN_VARIANTS, DROP_VARIANTS, WAITING_TO_START,
     WILDCASTLECHESS, PAWN, KNIGHT, SITTUYINCHESS, BLACK
 from pychess.Variants.blindfold import BlindfoldBoard, HiddenPawnsBoard, \
     HiddenPiecesBoard, AllWhiteBoard
-from .Background import hexcol
 from . import preferencesDialog
 
 
@@ -159,6 +161,7 @@ class BoardView(Gtk.DrawingArea):
             self.model.connect("game_started", self.gameStarted),
             self.model.connect("game_changed", self.gameChanged),
             self.model.connect("moves_undoing", self.movesUndoing),
+            self.model.connect("variation_undoing", self.variationUndoing),
             self.model.connect("game_loading", self.gameLoading),
             self.model.connect("game_loaded", self.gameLoaded),
             self.model.connect("game_ended", self.gameEnded),
@@ -174,17 +177,17 @@ class BoardView(Gtk.DrawingArea):
             conf.notify_add("showCords", self.onShowCords),
             conf.notify_add("showCaptured", self.onShowCaptured),
             conf.notify_add("faceToFace", self.onFaceToFace),
-            conf.notify_add("pieceTheme", self.onSetPieceTheme),
-            conf.notify_add("board_frame", self.onBoardFrameTheme),
-            conf.notify_add("board_style", self.onBoardColourTheme),
-            conf.notify_add("lightcolour", self.onBoardColourTheme),
-            conf.notify_add("darkcolour", self.onBoardColourTheme),
+            conf.notify_add("noAnimation", self.onNoAnimation),
+            conf.notify_add("autoRotate", self.onAutoRotate),
+            conf.notify_add("pieceTheme", self.onPieceTheme),
+            conf.notify_add("board_frame", self.onBoardFrame),
+            conf.notify_add("board_style", self.onBoardStyle),
+            conf.notify_add("lightcolour", self.onBoardColour),
+            conf.notify_add("darkcolour", self.onBoardColour),
         ]
         self.RANKS = self.model.boards[0].RANKS
         self.FILES = self.model.boards[0].FILES
 
-        self.animimation_id = -1
-        self._do_stop = False
         self.animation_start = time()
         self.last_shown = None
         self.deadlist = []
@@ -213,20 +216,25 @@ class BoardView(Gtk.DrawingArea):
 
         self.no_frame = False
         self._show_cords = False
-        self.show_cords = conf.get("showCords", True)
+        self.show_cords = conf.get("showCords")
 
         self._draw_grid = False
-        self.draw_grid = conf.get("drawGrid", True)
+        self.draw_grid = conf.get("drawGrid")
 
         self._show_captured = False
         if self.preview:
             self.showCaptured = False
         else:
-            self.showCaptured = conf.get("showCaptured", False) or \
+            self.showCaptured = conf.get("showCaptured") or \
                 self.model.variant.variant in DROP_VARIANTS
 
-        self.onBoardColourTheme()
-        self.onBoardFrameTheme()
+        self.noAnimation = conf.get("noAnimation")
+        self.faceToFace = conf.get("faceToFace")
+        self.autoRotate = conf.get("autoRotate")
+
+        self.onBoardColour()
+        self.onBoardStyle()
+        self.onBoardFrame()
 
         self._show_enpassant = False
 
@@ -254,6 +262,10 @@ class BoardView(Gtk.DrawingArea):
         self.pre_arrow = None
         self.pre_circle = None
 
+        # circles and arrows from .pgn comments
+        self.saved_circles = set()
+        self.saved_arrows = set()
+
     def _del(self):
         self.disconnect(self.draw_cid)
         self.disconnect(self.realize_cid)
@@ -263,7 +275,10 @@ class BoardView(Gtk.DrawingArea):
             self.model.disconnect(cid)
 
     def gameStarted(self, model):
-        if conf.get("noAnimation", False):
+        if model.lesson_game:
+            self.shown = model.lowply
+
+        if self.noAnimation:
             self.got_started = True
             self.redrawCanvas()
         else:
@@ -304,7 +319,7 @@ class BoardView(Gtk.DrawingArea):
             self.shown = ply
 
             # Rotate board
-            if conf.get("autoRotate", False):
+            if self.autoRotate:
                 if self.model.players and self.model.curplayer.__type__ == LOCAL:
                     self.rotation = self.model.boards[-1].color * pi
 
@@ -321,6 +336,9 @@ class BoardView(Gtk.DrawingArea):
             self.shown = model.ply - moves
         self.redrawCanvas()
 
+    def variationUndoing(self, model):
+        self.showPrev()
+
     def gameLoading(self, model, uri):
         self.auto_update_shown = False
 
@@ -363,37 +381,54 @@ class BoardView(Gtk.DrawingArea):
         """ Checks the configuration / preferences to see if the board
             grid should be displayed.
         """
-        self.draw_grid = conf.get("drawGrid", True)
+        self.draw_grid = conf.get("drawGrid")
 
     def onShowCords(self, *args):
         """ Checks the configuration / preferences to see if the board
             co-ordinates should be displayed.
         """
-        self.show_cords = conf.get("showCords", True)
+        self.show_cords = conf.get("showCords")
 
     def onShowCaptured(self, *args):
         """ Check the configuration / preferences to see if
-            cthe captured pieces should be displayed
+            the captured pieces should be displayed
         """
-        self.showCaptured = conf.get("showCaptured", False)
+        self.showCaptured = conf.get("showCaptured")
+
+    def onNoAnimation(self, *args):
+        """ Check the configuration / preferences to see if
+            no animation needed at all
+        """
+        self.noAnimation = conf.get("noAnimation")
 
     def onFaceToFace(self, *args):
         """ If the preference for pieces to be displayed facing each other
             has been set then refresh the board
         """
+        self.faceToFace = conf.get("faceToFace")
         self.redrawCanvas()
 
-    def onSetPieceTheme(self, *args):
+    def onAutoRotate(self, *args):
+        self.autoRotate = conf.get("autoRotate")
+
+    def onPieceTheme(self, *args):
         """ If the preference to display another chess set has been
             selected then refresh the board
         """
         self.redrawCanvas()
 
-    def onBoardColourTheme(self, *args):
+    def onBoardColour(self, *args):
         """ If the preference to display another set of board colours has been
             selected then refresh the board
         """
-        board_style = conf.get("board_style", 1)
+        self.light_colour = conf.get("lightcolour")
+        self.dark_colour = conf.get("darkcolour")
+
+    def onBoardStyle(self, *args):
+        """ If the preference to display another set of board colours has been
+            selected then refresh the board
+        """
+        board_style = conf.get("board_style")
         self.colors_only = board_style == 0
         if not self.colors_only:
             # create dark and light square surfaces
@@ -407,8 +442,8 @@ class BoardView(Gtk.DrawingArea):
 
         self.redrawCanvas()
 
-    def onBoardFrameTheme(self, *args):
-        board_frame = conf.get("board_frame", 1)
+    def onBoardFrame(self, *args):
+        board_frame = conf.get("board_frame")
         self.no_frame = board_frame == 0
         if not self.no_frame:
             # create board frame surface
@@ -456,7 +491,6 @@ class BoardView(Gtk.DrawingArea):
             adjust the shown variation index too.
             If board is in the main line, reset the shown variation idx to 0(the main line).
         """
-
         if board in self.model.variations[self.shown_variation_idx]:
             # if the board to be shown is in the current shown variation, we are ok
             self.shown = self.model.variations[self.shown_variation_idx].index(board) + \
@@ -483,24 +517,78 @@ class BoardView(Gtk.DrawingArea):
     def shownIsMainLine(self):
         return self.shown_variation_idx == 0
 
+    @property
+    def has_unsaved_shapes(self):
+        return self.saved_arrows != self.arrows or self.saved_circles != self.circles
+
     def _getShown(self):
         return self._shown
 
-    def _setShown(self, shown):
-        """Adjust the index in current variation board list."""
-
-        # We don't do anything if we are already showing the right ply
-        if shown == self._shown:
-            return
+    def _setShown(self, shown, old_variation_idx=None):
+        """ Adjust the index in current variation board list.
+            old_variation_index is used when variation was added
+            to the last played move and we want to step back.
+        """
+        assert shown >= 0
+        if shown < self.model.lowply:
+            shown = self.model.lowply
 
         # This would cause IndexErrors later
-        if not self.model.lowply <= shown <= self.model.variations[self.shown_variation_idx][-1].ply:
+        if shown > self.model.variations[self.shown_variation_idx][-1].ply:
             return
 
+        if old_variation_idx is None:
+            old_variation_idx = self.shown_variation_idx
+
         self.redarrow = None
         self.greenarrow = None
         self.bluearrow = None
 
+        # remove all circles and arrows
+        need_redraw = False
+        if self.saved_circles:
+            self.saved_circles.clear()
+            need_redraw = True
+        if self.saved_arrows:
+            self.saved_arrows.clear()
+            need_redraw = True
+        if self.arrows:
+            self.arrows.clear()
+            need_redraw = True
+        if self.circles:
+            self.circles.clear()
+            need_redraw = True
+        if self.pre_arrow is not None:
+            self.pre_arrow = None
+            need_redraw = True
+        if self.pre_circle is not None:
+            self.pre_circle = None
+            need_redraw = True
+
+        # search circles/arrows in move comments
+        board = self.model.getBoardAtPly(shown, self.shown_variation_idx).board
+        if board.children:
+            for child in board.children:
+                if isinstance(child, str):
+                    if "[%csl" in child:
+                        match = comment_circles_re.search(child)
+                        circles = match.groups()[0].split(",")
+                        for circle in circles:
+                            self.saved_circles.add(Cord(circle[1:3], color=circle[0]))
+                            self.circles.add(Cord(circle[1:3], color=circle[0]))
+                        need_redraw = True
+
+                    if "[%cal" in child:
+                        match = comment_arrows_re.search(child)
+                        arrows = match.groups()[0].split(",")
+                        for arrow in arrows:
+                            self.saved_arrows.add((Cord(arrow[1:3], color=arrow[0]), Cord(arrow[3:5])))
+                            self.arrows.add((Cord(arrow[1:3], color=arrow[0]), Cord(arrow[3:5])))
+                        need_redraw = True
+
+        if need_redraw:
+            self.redrawCanvas()
+
         # If there is only one board, we don't do any animation, but simply
         # redraw the entire board. Same if we are at first draw.
         if len(self.model.boards) == 1 or self.shown < self.model.lowply:
@@ -515,13 +603,13 @@ class BoardView(Gtk.DrawingArea):
 
         deadset = set()
         for i in range(self.shown, shown, step):
-            board = self.model.getBoardAtPly(i, self.shown_variation_idx)
+            board = self.model.getBoardAtPly(i, old_variation_idx)
             board1 = self.model.getBoardAtPly(i + step, self.shown_variation_idx)
             if step == 1:
                 move = self.model.getMoveAtPly(i, self.shown_variation_idx)
                 moved, new, dead = board.simulateMove(board1, move)
             else:
-                move = self.model.getMoveAtPly(i - 1, self.shown_variation_idx)
+                move = self.model.getMoveAtPly(i - 1, old_variation_idx)
                 moved, new, dead = board.simulateUnmove(board1, move)
 
             # We need to ensure, that the piece coordinate is saved in the
@@ -547,19 +635,17 @@ class BoardView(Gtk.DrawingArea):
                 piece.opacity = 0
 
         self.deadlist = []
-        for y_loc, row in enumerate(self.model.getBoardAtPly(self.shown,
-                                                             self.shown_variation_idx).data):
+        for y_loc, row in enumerate(self.model.getBoardAtPly(self.shown, old_variation_idx).data):
             for x_loc, piece in row.items():
                 if piece in deadset:
                     self.deadlist.append((piece, x_loc, y_loc))
 
         self._shown = shown
         if self.real_set_shown:
+            if self.model.getBoardAtPly(self.shown, self.shown_variation_idx) in self.model.variations[0]:
+                self.shown_variation_idx = 0
             self.emit("shownChanged", self.shown)
 
-        if self.animimation_id != -1:
-            self._do_stop = True
-
         self.animation_start = time()
         self.animating = True
 
@@ -567,18 +653,18 @@ class BoardView(Gtk.DrawingArea):
             paint_box = self.paintBoxAround(self.lastMove)
             self.lastMove = None
             self.redrawCanvas(rect(paint_box))
+
         if self.shown > self.model.lowply:
             self.lastMove = self.model.getMoveAtPly(self.shown - 1, self.shown_variation_idx)
+            paint_box = self.paintBoxAround(self.lastMove)
+            self.redrawCanvas(rect(paint_box))
         else:
             self.lastMove = None
 
-        def doSetShown():
-            self.runAnimation(redraw_misc=self.real_set_shown)
-            if not conf.get("noAnimation", False):
-                while self.animating:
-                    self.animimation_id = self.runAnimation()
-
-        doSetShown()
+        self.runAnimation(redraw_misc=self.real_set_shown)
+        if not self.noAnimation:
+            while self.animating:
+                self.runAnimation()
 
     shown = property(_getShown, _setShown)
 
@@ -594,13 +680,10 @@ class BoardView(Gtk.DrawingArea):
             which starts the animation, also sets a timestamp for the acceleration
             to work properply.
         """
-
         if self.model is None:
             return False
 
-        if self._do_stop:
-            self._do_stop = False
-            self.animimation_id = - 1
+        if not self.animating:
             return False
 
         paint_box = None
@@ -630,7 +713,7 @@ class BoardView(Gtk.DrawingArea):
                         y_loc = self.premove1.y
 
                 if piece.x is not None:
-                    if not conf.get("noAnimation", False):
+                    if not self.noAnimation:
                         if piece.piece == KNIGHT:
                             newx = piece.x + (x_loc - piece.x) * mod**(1.5)
                             newy = piece.y + (y_loc - piece.y) * mod
@@ -667,7 +750,7 @@ class BoardView(Gtk.DrawingArea):
                     else:
                         paint_box = self.cord2RectRelative(px_loc, py_loc)
 
-                    if not conf.get("noAnimation", False):
+                    if not self.noAnimation:
                         new_op = piece.opacity + (1 - piece.opacity) * mod
                     else:
                         new_op = 1
@@ -685,7 +768,7 @@ class BoardView(Gtk.DrawingArea):
             else:
                 paint_box = join(paint_box, self.cord2RectRelative(x_loc, y_loc))
 
-            if not conf.get("noAnimation", False):
+            if not self.noAnimation:
                 new_op = piece.opacity + (0 - piece.opacity) * mod
             else:
                 new_op = 0
@@ -701,7 +784,7 @@ class BoardView(Gtk.DrawingArea):
         if paint_box:
             self.redrawCanvas(rect(paint_box))
 
-        if conf.get("noAnimation", False):
+        if self.noAnimation:
             self.animating = False
             return False
         else:
@@ -710,15 +793,13 @@ class BoardView(Gtk.DrawingArea):
             return paint_box and True or False
 
     def startAnimation(self):
-        def doStartAnimation():
-            self.runAnimation(redraw_misc=True)
-            if not conf.get("noAnimation", False):
-                while self.animating:
-                    self.animimation_id = self.runAnimation()
-
         self.animation_start = time()
         self.animating = True
-        doStartAnimation()
+
+        self.runAnimation(redraw_misc=True)
+        if not self.noAnimation:
+            while self.animating:
+                self.runAnimation()
 
     #############################
     #          Drawing          #
@@ -768,17 +849,13 @@ class BoardView(Gtk.DrawingArea):
     ###############################
 
     def redrawCanvas(self, rect=None):
-
-        def redraw(rect):
-            if self.get_window():
-                if not rect:
-                    alloc = self.get_allocation()
-                    rect = Gdk.Rectangle()
-                    rect.x, rect.y, rect.width, rect.height = (0, 0, alloc.width, alloc.height)
-#                    self.queue_draw_area(r.x, r.y, r.width, r.height)
-                self.get_window().invalidate_rect(rect, True)
-                self.get_window().process_updates(True)
-        redraw(rect)
+        if self.get_window():
+            if not rect:
+                alloc = self.get_allocation()
+                rect = Gdk.Rectangle()
+                rect.x, rect.y, rect.width, rect.height = (0, 0, alloc.width, alloc.height)
+            self.get_window().invalidate_rect(rect, True)
+            self.get_window().process_updates(True)
 
     ###############################
     #            draw             #
@@ -861,7 +938,7 @@ class BoardView(Gtk.DrawingArea):
 
         xc_loc, yc_loc, square, side = self.square
 
-        if contains(rect((xc_loc, yc_loc, square)), rectangle):
+        if rectangle is not None and contains(rect((xc_loc, yc_loc, square)), rectangle):
             return
 
         thick = thickness * square
@@ -891,9 +968,6 @@ class BoardView(Gtk.DrawingArea):
                 layout.set_font_description(
                     Pango.FontDescription("bold %d" % sign_size))
 
-                width = layout.get_pixel_size()[0]
-                height = layout.get_pixel_size()[1]
-
                 # Draw bottom
                 context.move_to(xc_loc + side * num + side / 2 - width / 2, yc_loc + square)
                 PangoCairo.show_layout(context, layout)
@@ -906,7 +980,7 @@ class BoardView(Gtk.DrawingArea):
             paint(True)
             context.transform(invmatrix)
 
-        if conf.get("faceToFace", False):
+        if self.faceToFace:
             if self.rotation == 0:
                 context.transform(matrix)
                 paint(True)
@@ -951,16 +1025,18 @@ class BoardView(Gtk.DrawingArea):
     def drawBoard(self, context, r):
         xc_loc, yc_loc, square, side = self.square
 
-        style_ctxt = self.get_style_context()
-        LIGHT = hexcol(style_ctxt.lookup_color("p_light_color")[1])
         col = Gdk.RGBA()
-        col.parse(conf.get("lightcolour", LIGHT))
+        col.parse(self.light_colour)
         context.set_source_rgba(col.red, col.green, col.blue, col.alpha)
 
         if self.model.variant.variant in ASEAN_VARIANTS:
             # just fill the whole board with light color
-            context.rectangle(xc_loc, yc_loc, side * self.FILES, side * self.RANKS)
-            context.fill()
+            if self.colors_only:
+                context.rectangle(xc_loc, yc_loc, side * self.FILES, side * self.RANKS)
+            else:
+                self.draw_image(context, self.light_surface, xc_loc, yc_loc, side * self.FILES, side * self.RANKS)
+            if self.colors_only:
+                context.fill()
         else:
             # light squares
             for x_loc in range(self.FILES):
@@ -973,17 +1049,11 @@ class BoardView(Gtk.DrawingArea):
             if self.colors_only:
                 context.fill()
 
-        style_ctxt = self.get_style_context()
-        DARK = hexcol(style_ctxt.lookup_color("p_dark_color")[1])
         col = Gdk.RGBA()
-        col.parse(conf.get("darkcolour", DARK))
+        col.parse(self.dark_colour)
         context.set_source_rgba(col.red, col.green, col.blue, col.alpha)
 
         if self.model.variant.variant in ASEAN_VARIANTS:
-            # just unfilled rectangles
-            for x_loc in range(self.FILES):
-                for y_loc in range(self.RANKS):
-                    context.rectangle((xc_loc + x_loc * side), (yc_loc + y_loc * side), side, side)
             # diagonals
             if self.model.variant.variant == SITTUYINCHESS:
                 context.move_to(xc_loc, yc_loc)
@@ -1018,7 +1088,7 @@ class BoardView(Gtk.DrawingArea):
         if self.draw_grid:
             # grid lines between squares
             context.set_source_rgb(0.0, 0.0, 0.0)
-            context.set_line_width(1.0)
+            context.set_line_width(0.5 if r is None else 1.0)
 
             for loc in range(self.FILES):
                 context.move_to(xc_loc + side * loc, yc_loc)
@@ -1071,7 +1141,7 @@ class BoardView(Gtk.DrawingArea):
 
         side = self.square[3]
 
-        if not conf.get("faceToFace", False):
+        if not self.faceToFace:
             matrix, invmatrix, cx_loc, cy_loc = self.getCordMatrices(x_loc, y_loc)
         else:
             cx_loc, cy_loc = self.cord2Point(x_loc, y_loc)
@@ -1141,7 +1211,7 @@ class BoardView(Gtk.DrawingArea):
                     continue
                 if not piece or piece.x is not None or piece.opacity < 1:
                     continue
-                if not intersects(rect(self.cord2RectRelative(x_loc, y_loc)), rectangle):
+                if rectangle is not None and not intersects(rect(self.cord2RectRelative(x_loc, y_loc)), rectangle):
                     continue
                 if Cord(x_loc, y_loc) == self.selected:
                     context.set_source_rgb(*fg_s)
@@ -1220,18 +1290,32 @@ class BoardView(Gtk.DrawingArea):
                 context.set_source_rgba(red, green, blue, alpha)
             context.fill()
 
+    def color2rgba(self, color):
+        if color == "R":
+            rgba = (.643, 0, 0, 0.8)
+        elif color == "B":
+            rgba = (.204, .396, .643, 0.8)
+        elif color == "Y":
+            rgba = (.961, .475, 0, 0.8)
+        else:
+            # light_green
+            rgba = (0.337, 0.612, 0.117, 0.8)
+        return rgba
+
     def drawCircles(self, context):
         radius = self.square[3] / 2.0
         context.set_line_width(4)
-        light_green = (0.337, 0.612, 0.117, 0.8)
-        context.set_source_rgb(*light_green[:3])
 
         for cord in self.circles:
+            rgba = self.color2rgba(cord.color)
+            context.set_source_rgb(*rgba[:3])
             x_loc, y_loc = self.cord2Point(cord)
             context.new_sub_path()
             context.arc(x_loc + radius, y_loc + radius, radius - 3, 0, 2 * pi)
             context.stroke()
         if self.pre_circle is not None:
+            rgba = self.color2rgba(self.pre_circle.color)
+            context.set_source_rgb(*rgba[:3])
             x_loc, y_loc = self.cord2Point(self.pre_circle)
             context.new_sub_path()
             context.arc(x_loc + radius, y_loc + radius, radius - 3, 0, 2 * pi)
@@ -1242,11 +1326,11 @@ class BoardView(Gtk.DrawingArea):
         arhh = 0.6  # Arrow head height
         arsw = 0.0  # Arrow stroke width
         for arrow_cords in self.arrows:
-            self.__drawArrow(context, arrow_cords, arw, arhw, arhh, arsw,
-                             light_green, light_green)
+            rgba = self.color2rgba(arrow_cords[0].color)
+            self.__drawArrow(context, arrow_cords, arw, arhw, arhh, arsw, rgba, rgba)
         if self.pre_arrow is not None:
-            self.__drawArrow(context, self.pre_arrow, arw, arhw, arhh, arsw,
-                             light_green, light_green)
+            rgba = self.color2rgba(self.pre_arrow[0].color)
+            self.__drawArrow(context, self.pre_arrow, arw, arhw, arhh, arsw, rgba, rgba)
 
     ###############################
     #        drawLastMove         #
@@ -1627,7 +1711,7 @@ class BoardView(Gtk.DrawingArea):
     ################################
 
     def _setRotation(self, radians):
-        if not conf.get("fullAnimation", True):
+        if not conf.get("fullAnimation"):
             self._rotation = radians
             self.next_rotation = radians
             self.matrix = cairo.Matrix.init_rotate(radians)
@@ -1645,6 +1729,7 @@ class BoardView(Gtk.DrawingArea):
                 if amount > 1:
                     amount = 1
                     next = False
+                    self.animating = False
                 else:
                     next = True
                 self._rotation = new = oldr + amount * (radians - oldr)
@@ -1753,17 +1838,26 @@ class BoardView(Gtk.DrawingArea):
             self.shown_variation_idx = 0
 
     def showPrev(self, step=1):
+        # If prev board belongs to a higher level variation
+        # we have to update shown_variation_idx
+        old_variation_idx = None
+        if not self.shownIsMainLine():
+            board = self.model.getBoardAtPly(self.shown - step, self.shown_variation_idx)
+            for vari in self.model.variations:
+                if board in vari:
+                    break
+            # swich to the new variation
+            old_variation_idx = self.shown_variation_idx
+            self.shown_variation_idx = self.model.variations.index(vari)
+
         if self.model.examined and self.model.noTD:
             self.model.goPrev(step)
         else:
             if self.shown > self.model.lowply:
                 if self.shown - step > self.model.lowply:
-                    self.shown -= step
+                    self._setShown(self.shown - step, old_variation_idx)
                 else:
-                    self.shown = self.model.lowply
-
-                if self.model.getBoardAtPly(self.shown, self.shown_variation_idx) in self.model.variations[0]:
-                    self.shown_variation_idx = 0
+                    self._setShown(self.model.lowply, old_variation_idx)
 
     def showNext(self, step=1):
         if self.model.examined and self.model.noTD:
@@ -1796,3 +1890,13 @@ class BoardView(Gtk.DrawingArea):
         self.premove1 = premove1
         self.premove_ply = premove_ply
         self.premove_promotion = promotion
+
+    def copy_pgn(self):
+        output = StringIO()
+        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+        clipboard.set_text(pgn.save(output, self.model), -1)
+
+    def copy_fen(self):
+        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+        fen = self.model.getBoardAtPly(self.shown, self.shown_variation_idx).asFen()
+        clipboard.set_text(fen, -1)
