@@ -2,15 +2,13 @@
 
 import sys
 from collections import defaultdict
-from io import StringIO
 
-from gi.repository import Gdk, Gtk, GObject
+from gi.repository import Gtk, GObject
 
 import pychess
 from .BoardControl import BoardControl
 from .ChessClock import ChessClock
 from .MenuItemsDict import MenuItemsDict
-from pychess.Savers import pgn
 from pychess.System import conf
 
 from pychess.System.Log import log
@@ -94,7 +92,7 @@ class GameWidget(GObject.GObject):
         self.notify_cids = [conf.notify_add("showFICSgameno", self.on_show_fics_gameno), ]
 
         if self.gamemodel.display_text:
-            if isinstance(self.gamemodel, ICGameModel) and conf.get("showFICSgameno", False):
+            if isinstance(self.gamemodel, ICGameModel) and conf.get("showFICSgameno"):
                 self.game_info_label.set_text("%s [%s]" % (
                     self.display_text, self.gamemodel.ficsgame.gameno))
             else:
@@ -114,9 +112,8 @@ class GameWidget(GObject.GObject):
             self.gamemodel.gmwidg_ready.set()
 
     def _del(self):
-        if isinstance(self.gamemodel, ICGameModel):
-            if self.gamemodel.connection.offline_lecture:
-                self.gamemodel.connection.lecture_exit_event.set()
+        if self.gamemodel.offline_lecture:
+            self.gamemodel.lecture_exit_event.set()
 
         for obj in self.cids:
             if obj.handler_is_connected(self.cids[obj]):
@@ -145,7 +142,7 @@ class GameWidget(GObject.GObject):
         """ Checks the configuration / preferences to see if the FICS
             game number should be displayed next to player names.
         """
-        if isinstance(self.gamemodel, ICGameModel) and conf.get("showFICSgameno", False):
+        if isinstance(self.gamemodel, ICGameModel) and conf.get("showFICSgameno"):
             self.game_info_label.set_text(" [%s]" % self.gamemodel.ficsgame.gameno)
         else:
             self.game_info_label.set_text("")
@@ -291,6 +288,8 @@ class GameWidget(GObject.GObject):
             if self.gamemodel.timemodel.hasBWTimes(bmovecount, wmovecount):
                 self.clock.update(wmovecount, bmovecount)
 
+        self.on_shapes_changed(self.board)
+
     def game_started(self, gamemodel):
         if self.gamemodel.isLocalGame():
             self.menuitems["abort"].label = _("Abort")
@@ -359,6 +358,7 @@ class GameWidget(GObject.GObject):
         if gamemodel.isObservationGame() and not self.isInFront():
             self.light_on_off(True)
 
+        # print(gamemodel.waitingplayer, gamemodel.waitingplayer.__type__)
         if not gamemodel.isPlayingICSGame():
             self.clearMessages()
 
@@ -630,7 +630,7 @@ class GameWidget(GObject.GObject):
         for item in ACTION_MENU_ITEMS:
             actionMenuDic[item] = widgets[item]
 
-        if isinstance(self.gamemodel, ICGameModel) and self.gamemodel.connection.offline_lecture:
+        if self.gamemodel.offline_lecture:
             preview = True
         else:
             preview = False
@@ -663,12 +663,20 @@ class GameWidget(GObject.GObject):
         lastButton.set_tooltip_text(_("Jump to latest position"))
         toolbar.insert(lastButton, -1)
 
-        filterButton = Gtk.ToggleToolButton(Gtk.STOCK_FIND)
+        filterButton = Gtk.ToolButton(Gtk.STOCK_FIND)
         filterButton.set_tooltip_text(_("Find postion in current database"))
         toolbar.insert(filterButton, -1)
 
+        self.saveButton = Gtk.ToolButton(Gtk.STOCK_SAVE)
+        self.saveButton.set_tooltip_text(_("Save arrows/circles"))
+        toolbar.insert(self.saveButton, -1)
+
         def on_clicked(button, func):
-            func()
+            # Prevent moving in game while lesson not finished
+            if self.gamemodel.lesson_game and not self.gamemodel.solved:
+                return
+            else:
+                func()
 
         self.cids[firstButton] = firstButton.connect("clicked", on_clicked, self.board.view.showFirst)
         self.cids[prevButton] = prevButton.connect("clicked", on_clicked, self.board.view.showPrev)
@@ -676,6 +684,10 @@ class GameWidget(GObject.GObject):
         self.cids[nextButton] = nextButton.connect("clicked", on_clicked, self.board.view.showNext)
         self.cids[lastButton] = lastButton.connect("clicked", on_clicked, self.board.view.showLast)
         self.cids[filterButton] = filterButton.connect("clicked", on_clicked, self.find_in_database)
+        self.cids[self.saveButton] = self.saveButton.connect("clicked", on_clicked, self.save_shapes_to_pgn)
+
+        self.on_shapes_changed(self.board)
+        self.board.connect("shapes_changed", self.on_shapes_changed)
 
         tool_box = Gtk.Box()
         tool_box.pack_start(toolbar, True, True, 0)
@@ -683,6 +695,9 @@ class GameWidget(GObject.GObject):
         align.add(tool_box)
         return align
 
+    def on_shapes_changed(self, boardcontrol):
+        self.saveButton.set_sensitive(boardcontrol.view.has_unsaved_shapes)
+
     def find_in_database(self):
         persp = perspective_manager.get_perspective("database")
         if persp.chessfile is None:
@@ -705,6 +720,41 @@ class GameWidget(GObject.GObject):
             persp.gamelist.load_games()
             perspective_manager.activate_perspective("database")
 
+    def save_shapes_to_pgn(self):
+        view = self.board.view
+        shown_board = self.gamemodel.getBoardAtPly(view.shown, view.shown_variation_idx)
+
+        for child in shown_board.board.children:
+            if isinstance(child, str):
+                if child.lstrip().startswith("[%csl "):
+                    shown_board.board.children.remove(child)
+                    self.gamemodel.needsSave = True
+                elif child.lstrip().startswith("[%cal "):
+                    shown_board.board.children.remove(child)
+                    self.gamemodel.needsSave = True
+
+        if view.circles:
+            csl = []
+            for circle in view.circles:
+                csl.append("%s%s" % (circle.color, repr(circle)))
+            shown_board.board.children = ["[%%csl %s]" % ",".join(csl)] + shown_board.board.children
+            self.gamemodel.needsSave = True
+
+        if view.arrows:
+            cal = []
+            for arrow in view.arrows:
+                cal.append("%s%s%s" % (arrow[0].color, repr(arrow[0]), repr(arrow[1])))
+            shown_board.board.children = ["[%%cal %s]" % ",".join(cal)] + shown_board.board.children
+            self.gamemodel.needsSave = True
+
+        view.saved_arrows = set()
+        view.saved_arrows |= view.arrows
+
+        view.saved_circles = set()
+        view.saved_circles |= view.circles
+
+        self.on_shapes_changed(self.board)
+
     def light_on_off(self, on):
         child = self.tabcontent.get_child()
         if child:
@@ -765,18 +815,6 @@ class GameWidget(GObject.GObject):
         if self == self.perspective.cur_gmwidg():
             self.perspective.notebooks["messageArea"].hide()
 
-    def copy_pgn(self):
-        output = StringIO()
-        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
-        clipboard.set_text(pgn.save(output, self.gamemodel), -1)
-
-    def copy_fen(self):
-        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
-        ply = self.board.view.shown
-        variation = self.board.view.shown_variation_idx
-        fen = self.gamemodel.getBoardAtPly(ply, variation).asFen()
-        clipboard.set_text(fen, -1)
-
 
 # ###############################################################################
 # Handling of the special sidepanels-design-gamewidget used in preferences     #
